SyncProgress

Struct SyncProgress 

pub struct SyncProgress { /* private fields */ }
A structure that tracks the progression and the errors that happen during a sync

Implementations

Source

impl SyncProgress

pub fn new() -> Self

pub fn new_with_feedback_channel(channel: FeedbackSender) -> Self

pub fn reset_counter(&mut self)

Reset the user-info counter

pub fn increment_counter(&mut self, increment: usize)

Increments the user-info counter.

pub fn counter(&self) -> usize

Retrieves the current user-info counter. This counts “arbitrary things”, that’s provided as a convenience but it is not used internally (e.g. that can be used to keep track of the items handled for the current calendar)

pub fn is_success(&self) -> bool

pub fn error(&mut self, text: &str)

Log an error

pub fn warn(&mut self, text: &str)

Log a warning

pub fn info(&mut self, text: &str)

Log an info

pub fn debug(&mut self, text: &str)

Log a debug message

pub fn trace(&mut self, text: &str)

Log a trace message

pub fn feedback(&mut self, event: SyncEvent)

Send an event as a feedback to the listener (if any).