pub struct SyncProgress { /* private fields */ }Expand description
A structure that tracks the progression and the errors that happen during a sync
Implementations
Sourceimpl SyncProgress
impl SyncProgress
pub fn new() -> Self
pub fn new_with_feedback_channel(channel: FeedbackSender) -> Self
Sourcepub fn reset_counter(&mut self)
pub fn reset_counter(&mut self)
Reset the user-info counter
Sourcepub fn increment_counter(&mut self, increment: usize)
pub fn increment_counter(&mut self, increment: usize)
Increments the user-info counter.
Sourcepub fn counter(&self) -> usize
pub fn counter(&self) -> usize
Retrieves the current user-info counter. This counts “arbitrary things”, that’s provided as a convenience but it is not used internally (e.g. that can be used to keep track of the items handled for the current calendar)
pub fn is_success(&self) -> bool